@@ -19,14 +19,14 @@ double binomial(unsigned int n, unsigned int k, double p)
  double pnk;
 
  if(verbose > 10){
-   printf("binomial(): Making binomial p(%d,%d,%f)\n",n,k,p);
+   Rprintf("binomial(): Making binomial p(%d,%d,%f)\n",n,k,p);
  }
 
  pnk = gsl_sf_fact(n)*pow(p,(double)k)*pow((1.0-p),(double)(n-k))/
              (gsl_sf_fact(k)*gsl_sf_fact(n-k));
 
  if(verbose > 10){
-   printf("binomial(): Made binomial p(%d,%d,%f) = %f\n",n,k,p,pnk);
+   Rprintf("binomial(): Made binomial p(%d,%d,%f) = %f\n",n,k,p,pnk);
  }
 
  return(pnk);
